﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UmlDiagramDesigner.Models;

namespace UmlDiagramDesigner.Assets
{
    internal class Sender
    {
        public void SendMessage()
        {
            Messenger.Send("Data zpravy");
        }
    }

    internal class Reciever
    {
        public void RegisterMessage()
        {
            Messenger.Register<string>(this,message =>
            {
                //zde lze vykonat libovolný kód s přijatou
                //zprávou typu string
            });
        }
    }

    public static class Messenger
    {
        private static readonly Dictionary<Type, IList<ReceiverAction>> RegisteredRecipients;
        static Messenger()
        {
            RegisteredRecipients = new Dictionary<Type, IList<ReceiverAction>>();
        }

        public static void Register<T>(object receiver, Action<T> action)
        {
            IList<ReceiverAction> value;
            if (RegisteredRecipients.TryGetValue(typeof(T), out value))
            {
                value.Add(new ReceiverAction(receiver, o => action((T)o)));
            }
            else
            {
                var recipients = new List<ReceiverAction>() { new ReceiverAction(receiver, o => action((T)o)) };
                RegisteredRecipients.Add(typeof(T), recipients);
            }
        }

        public static void Unregister<T>(object receiver, Action<T> action)
        {
            IList<ReceiverAction> value;
            if (RegisteredRecipients.TryGetValue(typeof(T), out value))
            {
                for (int i = 0; i < value.Count; i++)
                {
                    if(value[i].Receiver == receiver)
                        value.RemoveAt(i);
                }
            }
        }

        public static void Send<T>(T message)
        {
            IList<ReceiverAction> value;
            if (RegisteredRecipients.TryGetValue(typeof(T), out value))
            {
                foreach (var receiverAction in value)
                {
                    receiverAction.Action.Invoke(message);
                }
            }
        }




        private class ReceiverAction
        {
            public ReceiverAction(object receiver, Action<object> action)
            {
                Receiver = receiver;
                Action = action;
            }
            public object Receiver { get; private set; }
            public Action<object> Action { get; private set; }
        }
    }
}
