﻿using System.Collections.Generic;
using System.IO;
using Microsoft.Win32;
using UmlDiagramDesigner.Core;
using UmlDiagramDesigner.Services;
using UmlDiagramDesigner.ViewModels;

namespace UmlDiagramDesigner.DAL.Serialization
{
    public abstract class SerializerBase : ISerializer
    {
        protected SaveFileDialog SaveFileDialog;
        protected OpenFileDialog OpenFileDialog;
        protected SerializerBase()
        {
            DialogService = new WpfDialogService();
        }

        public IDialogService DialogService { get; set; }

        protected abstract string FileFilter { get; }

        protected string GetNewFilePath()
        {
            SaveFileDialog = new SaveFileDialog() {Filter = FileFilter, AddExtension = true};

            var result = SaveFileDialog.ShowDialog();

            if (result.HasValue && result == true)
                return SaveFileDialog.FileName;
            return null;
        }

        protected string GetOpenFilePath()
        {
            OpenFileDialog = new OpenFileDialog() {Filter = FileFilter, AddExtension = true};

            var result = OpenFileDialog.ShowDialog();

            if (result.HasValue && result == true)
            {
                if(Path.HasExtension(OpenFileDialog.FileName))
                    return OpenFileDialog.FileName;
            }
            return null;
        }

        public abstract void Serialize(IEnumerable<DiagramEntityViewModelBase> diagram);

        public abstract IEnumerable<DiagramEntityViewModelBase> Deserialize();
    }
}
