﻿using System;
using System.Windows;
using System.Windows.Controls.Primitives;
using UmlDiagramDesigner.ViewModels.DiagramItems;

namespace UmlDiagramDesigner.Helpers.Adorners
{
    public class ResizeThumb : DiagramItemThumbBase
    {
        public ResizeThumb()
        {
            DragDelta += this.ResizeThumb_DragDelta;
        }

        private void ResizeThumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            var item = this.DataContext as DiagramItemViewModelBase;

            if (item != null)
            {
                double deltaVertical, deltaHorizontal;

                switch (this.VerticalAlignment)
                {
                    case VerticalAlignment.Bottom:
                        deltaVertical = Math.Min(-e.VerticalChange, item.Height - item.MinHeight);
                        item.Height -= deltaVertical;
                        break;
                    case VerticalAlignment.Top:
                        deltaVertical = Math.Min(e.VerticalChange, item.Height - item.MinHeight);
                        item.TopPosition += deltaVertical;
                        item.Height -= deltaVertical;
                        break;
                }

                switch (HorizontalAlignment)
                {
                    case HorizontalAlignment.Left:
                        deltaHorizontal = Math.Min(e.HorizontalChange,
                            item.Width - item.MinWidth);
                        item.LeftPosition += deltaHorizontal;
                        item.Width -= deltaHorizontal;
                        break;
                    case HorizontalAlignment.Right:
                        deltaHorizontal = Math.Min(-e.HorizontalChange,
                            item.Width - item.MinWidth);
                        item.Width -= deltaHorizontal;
                        break;
                }
            }

            e.Handled = true;
        }
    }
}
