﻿using System.Windows;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using UmlDiagramDesigner.Properties;
using UmlDiagramDesigner.ViewModels;

namespace UmlDiagramDesigner.Helpers.Adorners
{
    public class SelectionBoxAdorner : Adorner
    {
        private Point? _startPoint, _endPoint;
        private readonly Pen _stroke;
        private readonly SolidColorBrush _fillBrush = new SolidColorBrush(Colors.DeepSkyBlue){Opacity = 0.2};
        private readonly CanvasViewModel _canvasViewModel;

        public SelectionBoxAdorner([NotNull] UIElement diagramCanvas, Point? startPoint, CanvasViewModel viewModel): base(diagramCanvas)
        {
            _startPoint = startPoint;
            _stroke = new Pen(Brushes.Black,0.8);
            _canvasViewModel = viewModel;
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
            {
                if (!IsMouseCaptured)
                    CaptureMouse();

                _endPoint = e.GetPosition(base.AdornedElement);
                InvalidateVisual();
            }
            else
            {
                if(IsMouseCaptured)
                    ReleaseMouseCapture();
                RemoveAllAdorners(AdornerLayer.GetAdornerLayer(AdornedElement));
            }


            e.Handled = true;
        }

        protected override void OnMouseUp(MouseButtonEventArgs e)
        {
            if(IsMouseCaptured)
                ReleaseMouseCapture();

            var adornerLayer = AdornerLayer.GetAdornerLayer(base.AdornedElement);
            RemoveAllAdorners(adornerLayer);

            if(SelectedRectangle.HasValue)
                _canvasViewModel.SelectItemsInSelectionBoxCommand.Execute(SelectedRectangle.Value);

            if(!base.AdornedElement.IsFocused)
                base.AdornedElement.Focus(); //focus to register events after selection (like keybindings)

            e.Handled = true;
        }

        protected override void OnRender(DrawingContext drawingContext)
        {
            base.OnRender(drawingContext);
            
            drawingContext.DrawRectangle(Brushes.Transparent, null, new Rect(RenderSize)); //just to be able to catch local events

            if(_startPoint.HasValue && _endPoint.HasValue)
                drawingContext.DrawRectangle(_fillBrush,_stroke,new Rect(_startPoint.Value,_endPoint.Value)); //draw real selection box
        }

        private void RemoveAllAdorners(AdornerLayer layer)
        {
            layer.Remove(this);
        }

        public Rect? SelectedRectangle
        {
            get
            {
                if (!_startPoint.HasValue || !_endPoint.HasValue) return null;
                return new Rect(_startPoint.Value,_endPoint.Value);
            }
        }
    }
}
