﻿using System;
using System.Windows;
using System.Windows.Controls;
using UmlDiagramDesigner.ViewModels.DiagramItems;
using UmlDiagramDesigner.ViewModels.DiagramRelationships;

namespace UmlDiagramDesigner.Helpers.Selectors
{
    class DiagramItemStyleSelector : StyleSelector
    {
        private static readonly ResourceDictionary DiagramItemsResourceDictionary;
        private static readonly ResourceDictionary DiagramRelationshipsResourceDictionary;
        private const string DiagramItemsResourceDictionaryPath = @"UmlDiagramDesigner;component/Resources/Styles/DiagramItemStyles.xaml";
        private const string DiagramRelationshipsResourceDictionaryPath =
            @"UmlDiagramDesigner;component/Resources/Styles/DiagramRelationshipStyles.xaml";

        static DiagramItemStyleSelector()
        {
            Instance = new DiagramItemStyleSelector();
            Uri tempUri;
            if (Uri.TryCreate(DiagramItemsResourceDictionaryPath, UriKind.RelativeOrAbsolute, out tempUri))
                DiagramItemsResourceDictionary = new ResourceDictionary()
                {
                    Source = tempUri
                };
            if (Uri.TryCreate(DiagramRelationshipsResourceDictionaryPath, UriKind.RelativeOrAbsolute, out tempUri))
                DiagramRelationshipsResourceDictionary = new ResourceDictionary
                {
                    Source =  tempUri
                };
        }

        public static DiagramItemStyleSelector Instance { get; private set; }

        public override Style SelectStyle(object item, DependencyObject container)
        {
            if (DiagramItemsResourceDictionary == null) return null;

            var diagramCanvas = ItemsControl.ItemsControlFromItemContainer(container);
            if (diagramCanvas == null) return null;

            var diagramItem = item as DiagramItemViewModelBase;

            if (diagramItem != null && DiagramItemsResourceDictionary.Contains(diagramItem.GetResourceStyleName()))
                return (Style)DiagramItemsResourceDictionary[diagramItem.GetResourceStyleName()];

            var diagramRelationship = item as RelationshipViewModelBase;

            if (diagramRelationship != null &&
                DiagramRelationshipsResourceDictionary.Contains(diagramRelationship.GetResourceStyleName()))
                return (Style) DiagramRelationshipsResourceDictionary[diagramRelationship.GetResourceStyleName()];

            return null;
        }
    }
}
