﻿using System.Collections.Generic;
using System.Linq;
using UmlDiagramDesigner.Core;
using UmlDiagramDesigner.Core.Graph;
using UmlDiagramDesigner.Models.Connectors;
using UmlDiagramDesigner.ViewModels.DiagramItems;

namespace UmlDiagramDesigner.Models
{
    public class ConnectorEngine : IConnectorEngine
    {
        public ConnectorEngine(IAssociationConnector<DiagramItemViewModelBase> associationConnector)
        {
            AssociationConnector = associationConnector;
        }

        /// <summary>
        /// Gets or sets the association connector. Strategy of connecting (or not connecting)
        /// relationships together is done by this property.
        /// </summary>
        /// <value>
        /// The association connector.
        /// </value>
        public IAssociationConnector<DiagramItemViewModelBase> AssociationConnector { get; set; }

        /// <summary>
        /// Updates connections of provided item.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <remarks>All neighbour relationships are taken into consideration.</remarks>
        public void UpdateConnections(DiagramItemViewModelBase item)
        {
            IEnumerable<IAssociation<DiagramItemViewModelBase>> neighborAssociations = null;

            foreach (var relationship in item.Relationships)
            {
                if (neighborAssociations == null)
                {
                    neighborAssociations = relationship.Target.Relationships;
                    neighborAssociations = neighborAssociations.Union(relationship.Source.Relationships);
                }
                neighborAssociations = neighborAssociations.Union(relationship.Source.Relationships);
                neighborAssociations = neighborAssociations.Union(relationship.Target.Relationships);
            }


            if (neighborAssociations != null)
            {
                AssociationConnector.Connect(neighborAssociations.ToArray());
            }
        }

        /// <summary>
        /// Updates the connections for the collection of items. Use this for example when loading graph from database.
        /// </summary>
        /// <param name="items">The items.</param>
        public void UpdateConnections(IEnumerable<DiagramItemViewModelBase> items)
        {
            foreach (var item in items)
                UpdateConnections(item);
        }
    }
}
