﻿using System;
using System.Runtime.Remoting.Channels;
using UmlDiagramDesigner.Core.Enums;

namespace UmlDiagramDesigner.ViewModels.DependentElements
{
    public class AttributeViewModel : ViewModelBase, IComparable<AttributeViewModel>
    {
        private string _name;
        private DbDataType _dataType;
        private bool _isPrimaryKey;
        private bool _isNotNull;
        private bool _isNullable;
        private bool _isAutoGenerated;
        private bool _isForeignKey;

        public AttributeViewModel()
        {
            Name = "AttributeName";
            DataType = DbDataType.Integer;
        }

        #region Properties

        public string Name
        {
            get { return _name; }
            set { _name = value; OnPropertyChanged(); OnPropertyChanged("FormattedString"); RaiseAttributeModifiedEvent(); }
        }

        public string AttributeCharacter
        {
            get { return (IsPrimaryKey || IsForeignKey) ? "" : ""; }
        }


        public DbDataType DataType
        {
            get { return _dataType; }
            set { _dataType = value; OnPropertyChanged(); OnPropertyChanged("FormattedString"); RaiseAttributeModifiedEvent(); }
        }

        public bool IsPrimaryKey
        {
            get { return _isPrimaryKey; }
            set { _isPrimaryKey = value; OnPropertyChanged(); OnPropertyChanged("AttributeCharacter"); OnPropertyChanged("FormattedString"); }
        }

        public bool IsNotNull
        {
            get { return _isNotNull; }
            set
            {
                _isNotNull = value;
                if(value)
                    _isNullable = false;

                OnPropertyChanged();
                OnPropertyChanged("FormattedString");
                OnPropertyChanged("IsNullable");
            }
        }

        public bool IsNullable
        {
            get { return _isNullable; }
            set
            {
                _isNullable = value;
                if(value)
                    _isNotNull = false;

                OnPropertyChanged();
                OnPropertyChanged("FormattedString");
                OnPropertyChanged("IsNotNull");
            }
        }

        public bool IsAutoGenerated
        {
            get { return _isAutoGenerated; }
            set { _isAutoGenerated = value; OnPropertyChanged(); OnPropertyChanged("FormattedString"); }
        }

        public bool IsForeignKey
        {
            get { return _isForeignKey; }
            set { _isForeignKey = value; OnPropertyChanged(); OnPropertyChanged("FormattedString"); OnPropertyChanged("AttributeCharacter"); }
        }

        public string FormattedString { get { return ToString(); }}

        #endregion

        private void RaiseAttributeModifiedEvent()
        {
            if (AttributeModified != null)
                AttributeModified(this, EventArgs.Empty);
        }

        public bool IsValid()
        {
            if (string.IsNullOrEmpty(Name))
                return false;

            return true;
        }

        public int CompareTo(AttributeViewModel other)
        {
            if (other == null) return 1;

            if (!other.IsPrimaryKey && IsPrimaryKey) return 1;
            if (other.IsPrimaryKey && IsPrimaryKey) return 0;
            if (other.IsPrimaryKey && !IsPrimaryKey) return -1;

            if (!other.IsForeignKey && IsForeignKey) return 1;
            if (other.IsForeignKey && IsForeignKey) return 0;
            if (other.IsForeignKey && !IsForeignKey) return -1;

            return 0;
        }

        public override string ToString()
        {
            var temp = string.Format("{0} : {1}", Name, DataType.ToString().ToUpper());

            if (IsPrimaryKey)
            {
                temp = string.Format("{0} <<PK>>", temp);
            }

            if (IsNotNull)
            {
                temp = string.Format("{0} <<{1}>>", temp, "Not Null");
            }

            if (IsNullable)
            {
                temp = string.Format("{0} <<{1}>>", temp, "Nullable");
            }

            if (IsAutoGenerated)
            {
                temp = string.Format("{0} <<{1}>>", temp, "Auto Generated");
            }

            if (IsForeignKey)
            {
                temp = string.Format("{0} <<FK>>", temp);
            }

            return temp;
        }

        public event EventHandler AttributeModified;
    }
}
