﻿using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows;
using UmlDiagramDesigner.Core;
using UmlDiagramDesigner.Properties;
using UmlDiagramDesigner.Services;
using UmlDiagramDesigner.ViewModels.Commands;

namespace UmlDiagramDesigner.ViewModels.InputForms
{
    public abstract class InputFormViewModelBase : INotifyPropertyChanged
    {
        protected IDialogService DialogService = new WpfDialogService();
        private string _errorText;

        public string ErrorText
        {
            get { return _errorText; }
            set
            {
                if (value == _errorText) return;
                _errorText = value;
                OnPropertyChanged();
            }
        }

        public GenericCommand<Window> OkCommand { get
        {
            return new GenericCommand<Window>(window =>
            {
                if (ValidateInputData())
                {
                    window.Close();
                    RaiseInputCompletedEvent();
                }
            });
        }}

        public GenericCommand<Window> CancelCommand
        {
            get
            {
                return new GenericCommand<Window>(window =>
                {
                    if (CanCancel())
                    {
                        window.Close();
                    }
                });
            }
        }

        private void RaiseInputCompletedEvent()
        {
            if (InputCompleted != null)
                InputCompleted(this, null);
        }

        protected bool SetErrorText(string error)
        {
            ErrorText = error;
            return false;
        }

        protected virtual bool CanCancel()
        {
            //default value, can override and do some logic to prevent cancel
            return true;
        }

        protected abstract bool ValidateInputData();

        public event EventHandler  InputCompleted;
        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
