﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text.RegularExpressions;
using UmlDiagramDesigner.Annotations;
using UmlDiagramDesigner.DTO;

namespace UmlDiagramDesigner.ViewModels.InputForms
{
    public class ViewEditViewModel : InputFormViewModelBase
    {
        private ObservableCollection<AttributeBySourceDTO> _attributes;
        private string _name;

        public ViewEditViewModel([NotNull] IEnumerable<AttributeBySourceDTO> groupedAttributes)
        {
            GroupedAttributes = new ObservableCollection<AttributeBySourceDTO>(groupedAttributes);
        }

        #region Properties

        public string Name
        {
            get { return _name; }
            set { _name = value; OnPropertyChanged();}
        }

        public ObservableCollection<AttributeBySourceDTO> GroupedAttributes
        {
            get { return _attributes; }
            set { _attributes = value; OnPropertyChanged();}
        }

        #endregion

        protected override bool ValidateInputData()
        {
            if (string.IsNullOrEmpty(Name))
                return SetErrorText("Name cannot be empty");

            if (!Regex.IsMatch(Name, @"^[a-zA-Z_0-9]+$"))
                return SetErrorText(
                    "Name can contain only letters from english alphabet or numbers (no spaces or special characters)");

            return true;
        }
    }
}
