﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UmlDiagramDesigner.CodeGeneration.Contracts;
using UmlDiagramDesigner.CodeGeneration.Enums;
using UmlDiagramDesigner.Core.Enums;
using ForeignKeyConstraint = UmlDiagramDesigner.Data.ForeignKeyConstraint;

namespace UmlDiagramDesigner.CodeGeneration
{
	/// <summary>
	/// Concrete properties for C# language.
	/// </summary>
	public class CSharpProperties : IOopLanguageProperties
    {
        private Dictionary<DbDataType, string> _dataTypes;
        private Dictionary<LanguageFeature, string> _features;
        private Dictionary<Accessibility, string> _accessibilities;

        public Dictionary<DbDataType, string> DataTypes
        {
            get
            {
                return _dataTypes ?? (_dataTypes = new Dictionary<DbDataType, string>()
                {
                    {DbDataType.BigInteger, "long"},
                    {DbDataType.Binary, "Int16"},
                    {DbDataType.Boolean, "Int16"},
                    {DbDataType.Char, "string"},
                    {DbDataType.Date, "DateTime"},
                    {DbDataType.Decimal, "decimal"},
                    {DbDataType.Float, "float"},
                    {DbDataType.Integer, "int"},
                    {DbDataType.SmallInteger, "Int16"},
                    {DbDataType.Time, "TimeSpan"},
                    {DbDataType.Timestamp, "object"}, //not supported in c#
                    {DbDataType.Varchar, "string"},
                });
            }
        }

        public string GenerateNavPropertyDataType(ForeignKeyConstraint navigationProperty)
        {
            switch (navigationProperty.TargetMultiplicity)
            {
                case MultiplicityType.One:
                    return navigationProperty.References.PartOf.Name;
                case MultiplicityType.ZeroToOne:
                    return navigationProperty.References.PartOf.Name;
                case MultiplicityType.ZeroToMany:
                    return string.Format("ICollection<{0}>", navigationProperty.References.PartOf.Name);
                case MultiplicityType.OneToMany:
                    return string.Format("ICollection<{0}>", navigationProperty.References.PartOf.Name);
                default:
                    throw new ArgumentOutOfRangeException("multiplicity");
            }
        }


        public Dictionary<LanguageFeature, string> Features
        {
            get { return _features ??(_features = new Dictionary<LanguageFeature, string>()
            {
                {LanguageFeature.BlockEnd, "}"},
                {LanguageFeature.BlockStart, "{"},
                {LanguageFeature.Comment, "//"},
                {LanguageFeature.EndLineCharacter, ";"},
                {LanguageFeature.Namespace, "namespace"},
                {LanguageFeature.NamespaceImport, "using"},
                {LanguageFeature.Inheritance, ":"},
            });}
        }

        public Dictionary<Accessibility, string> Accessibilities
        {
            get
            {
                return _accessibilities ?? (_accessibilities = new Dictionary<Accessibility, string>()
                {
                    {Accessibility.Private, "private"},
                    {Accessibility.Public, "public"},
                    {Accessibility.Protected, "protected"}
                });
            }
        }
    }
}
