﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UmlDiagramDesigner.CodeGeneration.Contracts;
using UmlDiagramDesigner.CodeGeneration.Enums;
using UmlDiagramDesigner.Core.Enums;
using UmlDiagramDesigner.Data;

namespace UmlDiagramDesigner.CodeGeneration
{
	/// <summary>
	/// Concrete properties for Microsoft SQL server.
	/// </summary>
	public class MsSqlProperties : ISqlProperties
    {
        private Dictionary<DbDataType, string> _dataTypes;
        private Dictionary<LanguageFeature, string> _features;
        private HashSet<string> _keywords;
        private Dictionary<TriggerEvent, string> _triggerEvents;
	    private Dictionary<Type, string> _constraints;

	    public Dictionary<LanguageFeature, string> Features
        {
            get
            {
                return _features ?? (_features = new Dictionary<LanguageFeature, string>()
                {
                    {LanguageFeature.Comment, "--"},
                    {LanguageFeature.EscapeCharacter, "\""}
                });
            }
        }

        public Dictionary<DbDataType, string> DataTypes
        {
            get { return _dataTypes ??(_dataTypes = new Dictionary<DbDataType, string>()
            {
                {DbDataType.BigInteger, "bigint"},
                {DbDataType.Binary, "binary(1000)"},
                {DbDataType.Boolean, "bit"},
                {DbDataType.Char, "nvarchar(max)"},
                {DbDataType.Date, "datetime2"},
                {DbDataType.Decimal, "real"},
                {DbDataType.Float, "float"},
                {DbDataType.Integer, "int"},
                {DbDataType.SmallInteger, "smallint"},
                {DbDataType.Time, "time"},
                {DbDataType.Timestamp, "timestamp"},
                {DbDataType.Varchar, "varchar(max)"},
            }); }
        }

        public Dictionary<TriggerEvent, string> TriggerEvents
        {
            get
            {
                return _triggerEvents ?? (_triggerEvents = new Dictionary<TriggerEvent, string>()
                {
                    {TriggerEvent.AfterDelete, "AFTER DELETE"},
                    {TriggerEvent.AfterInsert, "AFTER INSERT"},
                    {TriggerEvent.AfterUpdate, "AFTER UPDATE"},
                    {TriggerEvent.BeforeDelete, "AFTER DELETE"},
                    {TriggerEvent.BeforeInsert, "AFTER INSERT"},
                    {TriggerEvent.BeforeUpdate, "AFTER UPDATE"},
                });
            }
        }

	    public string Constraints<TConstraint>() where TConstraint : Constraint
	    {
		    if(_constraints == null)
				_constraints = new Dictionary<Type, string>()
				{
					{typeof(NotNullConstraint), "NOT NULL"},
					{typeof(NullableConstraint), "NULLABLE"},
					{typeof(UniqueConstraint), "UNIQUE"},
				};
		    return _constraints[typeof(TConstraint)];
	    }

		public HashSet<string> Keywords
        {
			get
			{
				if(_keywords != null) return _keywords;

				_keywords = new HashSet<string>();
				_keywords.UnionWith(new []
				{
					"ADD",
					"EXTERNAL",
					"PROCEDURE",
					"ALL",
					"FETCH",
					"PUBLIC",
					"ALTER",
					"FILE",
					"RAISERROR",
					"AND",
					"FILLFACTOR",
					"READ",
					"ANY",
					"FOR",
					"READTEXT",
					"AS",
					"FOREIGN",
					"RECONFIGURE",
					"ASC",
					"FREETEXT",
					"REFERENCES",
					"AUTHORIZATION",
					"FREETEXTTABLE",
					"REPLICATION",
					"BACKUP",
					"FROM",
					"RESTORE",
					"BEGIN",
					"FULL",
					"RESTRICT",
					"BETWEEN",
					"FUNCTION",
					"RETURN",
					"KEY",
					"ORDER",
					"BY",
					"KILL",
					"CONVERT",
					"SELECT",
					"ELSE",
					"PRINT"
				});
				return _keywords;
			}
        }
    }
}