﻿using System;
using System.Collections.Generic;
using UmlDiagramDesigner.Core.Graph;

namespace UmlDiagramDesigner.Core
{
	/// <summary>
	/// Provides an interface for abstract dialog service.
	/// Includes basic messages to display text, error or check user input.
	/// </summary>
	public interface IDialogService
    {
		/// <summary>
		/// Displays the specified message.
		/// </summary>
		/// <param name="message">The message.</param>
		void Display(string message);
		/// <summary>
		/// Displays the specified exception.
		/// </summary>
		/// <param name="exception">The exception.</param>
		void Display(Exception exception);
		/// <summary>
		/// Displays the specified exception with additional info.
		/// </summary>
		/// <param name="exception">The exception.</param>
		/// <param name="additionalInfo">The additional information.</param>
		void Display(Exception exception, string additionalInfo);
		/// <summary>
		/// Asks the specified question.
		/// </summary>
		/// <param name="question">The question.</param>
		/// <returns>Answer to the question</returns>
		bool Ask(string question);
    }
}
