﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UmlDiagramDesigner.Core.Graph;

namespace UmlDiagramDesigner.Data
{
	/// <summary>
	/// UML class
	/// </summary>
	public class Class : IIndexedVertex
    {
        public Class()
        {
            Children = new List<Class>();
        }

		/// <summary>
		/// Gets or sets the name.
		/// </summary>
		/// <value>
		/// The name.
		/// </value>
		public string Name { get; set; }
		/// <summary>
		/// Gets or sets the primary key.
		/// </summary>
		/// <value>
		/// The primary key.
		/// </value>
		public PrimaryKey PrimaryKey { get; set; }
		/// <summary>
		/// Gets or sets the ancestor (this class is a subtype of ancestor). 
		/// </summary>
		/// <value>
		/// The ancestor.
		/// </value>
		public Class Ancestor { get; set; }
		/// <summary>
		/// Gets or sets the children. Children are classes which inherit from this class.
		/// </summary>
		/// <value>
		/// The children.
		/// </value>
		public ICollection<Class> Children { get; set; }
		/// <summary>
		/// Gets or sets the attributes.
		/// </summary>
		/// <value>
		/// The attributes.
		/// </value>
		public ICollection<DataAttribute> Attributes { get; set; }
		/// <summary>
		/// Gets or sets the composition relationships.
		/// </summary>
		/// <value>
		/// The compositions.
		/// </value>
		public ICollection<ForeignKeyConstraint> Compositions { get; set; }
		/// <summary>
		/// Gets or sets the aggregation relationships.
		/// </summary>
		/// <value>
		/// The aggregations.
		/// </value>
		public ICollection<ForeignKeyConstraint> Aggregations { get; set; }
		/// <summary>
		/// Gets or sets the association relationships.
		/// </summary>
		/// <value>
		/// The associations.
		/// </value>
		public ICollection<ForeignKeyConstraint> Associations { get; set; }
		/// <summary>
		/// Gets or sets the triggers.
		/// </summary>
		/// <value>
		/// The triggers.
		/// </value>
		public ICollection<Trigger> Triggers { get; set; }

        #region IIndexedVertex implementation

        public int Index { get; set; }

        public int LowLink { get; set; }

        public IEnumerable<IIndexedVertex> GetIndexedNeighbours(bool revert = false)
        {
            if (revert)
            {
                return new[] {Ancestor};
            }
            return Children;
        }

        public IEnumerable<IIndexedVertex> GetIndexedNeighbours<T>(bool revert = false) where T : class
        {
            return GetIndexedNeighbours(revert);
        }

        public IEnumerable<IVertex> GetNeighbours(bool revert = false)
        {
            return GetIndexedNeighbours(revert);
        }

        public IEnumerable<IVertex> GetNeighbours<T>(bool revert = false) where T : class
        {
            return GetIndexedNeighbours(revert);
        }

        #endregion
    }
}
